@@ -1,4 +1,4 @@
-/* $OpenBSD: ssl_both.c,v 1.27 2021/03/29 16:46:09 jsing Exp $ */
+/* $OpenBSD: ssl_both.c,v 1.35 2021/09/03 13:19:12 jsing Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -118,14 +118,14 @@
 #include <stdio.h>
 #include <string.h>
 
-#include "ssl_locl.h"
-
 #include <openssl/buffer.h>
 #include <openssl/evp.h>
 #include <openssl/objects.h>
 #include <openssl/x509.h>
 
 #include "bytestring.h"
+#include "dtls_locl.h"
+#include "ssl_locl.h"
 
 /*
  * Send s->internal->init_buf in records of type 'type' (SSL3_RT_HANDSHAKE or
@@ -150,10 +150,8 @@ ssl3_do_write(SSL *s, int type)
 		    (unsigned char *)&s->internal->init_buf->data[s->internal->init_off], ret);
 
 	if (ret == s->internal->init_num) {
-		if (s->internal->msg_callback)
-			s->internal->msg_callback(1, s->version, type, s->internal->init_buf->data,
-			    (size_t)(s->internal->init_off + s->internal->init_num), s,
-			    s->internal->msg_callback_arg);
+		ssl_msg_callback(s, 1, type, s->internal->init_buf->data,
+		    (size_t)(s->internal->init_off + s->internal->init_num));
 		return (1);
 	}
 
@@ -164,42 +162,39 @@ ssl3_do_write(SSL *s, int type)
 }
 
 int
-ssl3_send_finished(SSL *s, int a, int b, const char *sender, int slen)
+ssl3_send_finished(SSL *s, int state_a, int state_b)
 {
 	CBB cbb, finished;
-	int md_len;
 
 	memset(&cbb, 0, sizeof(cbb));
 
-	if (S3I(s)->hs.state == a) {
-		md_len = TLS1_FINISH_MAC_LENGTH;
-		OPENSSL_assert(md_len <= EVP_MAX_MD_SIZE);
-
-		if (tls1_final_finish_mac(s, sender, slen,
-		    S3I(s)->hs.finished) != md_len)
-			return (0);
-		S3I(s)->hs.finished_len = md_len;
+	if (S3I(s)->hs.state == state_a) {
+		if (!tls12_derive_finished(s))
+			goto err;
 
 		/* Copy finished so we can use it for renegotiation checks. */
 		if (!s->server) {
 			memcpy(S3I(s)->previous_client_finished,
-			    S3I(s)->hs.finished, md_len);
-			S3I(s)->previous_client_finished_len = md_len;
+			    S3I(s)->hs.finished, S3I(s)->hs.finished_len);
+			S3I(s)->previous_client_finished_len =
+			    S3I(s)->hs.finished_len;
 		} else {
 			memcpy(S3I(s)->previous_server_finished,
-			    S3I(s)->hs.finished, md_len);
-			S3I(s)->previous_server_finished_len = md_len;
+			    S3I(s)->hs.finished, S3I(s)->hs.finished_len);
+			S3I(s)->previous_server_finished_len =
+			    S3I(s)->hs.finished_len;
 		}
 
 		if (!ssl3_handshake_msg_start(s, &cbb, &finished,
 		    SSL3_MT_FINISHED))
                         goto err;
-		if (!CBB_add_bytes(&finished, S3I(s)->hs.finished, md_len))
+		if (!CBB_add_bytes(&finished, S3I(s)->hs.finished,
+		    S3I(s)->hs.finished_len))
 			goto err;
 		if (!ssl3_handshake_msg_finish(s, &cbb))
 			goto err;
 
-		S3I(s)->hs.state = b;
+		S3I(s)->hs.state = state_b;
 	}
 
 	return (ssl3_handshake_write(s));
@@ -210,36 +205,6 @@ ssl3_send_finished(SSL *s, int a, int b, const char *sender, int slen)
 	return (-1);
 }
 
-/*
- * ssl3_take_mac calculates the Finished MAC for the handshakes messages seen
- * so far.
- */
-static void
-ssl3_take_mac(SSL *s)
-{
-	const char *sender;
-	int slen;
-
-	/*
-	 * If no new cipher setup return immediately: other functions will
-	 * set the appropriate error.
-	 */
-	if (S3I(s)->hs.cipher == NULL)
-		return;
-
-	if (S3I(s)->hs.state & SSL_ST_CONNECT) {
-		sender = TLS_MD_SERVER_FINISH_CONST;
-		slen = TLS_MD_SERVER_FINISH_CONST_SIZE;
-	} else {
-		sender = TLS_MD_CLIENT_FINISH_CONST;
-		slen = TLS_MD_CLIENT_FINISH_CONST_SIZE;
-	}
-
-	S3I(s)->hs.peer_finished_len =
-	    tls1_final_finish_mac(s, sender, slen,
-		S3I(s)->hs.peer_finished);
-}
-
 int
 ssl3_get_finished(SSL *s, int a, int b)
 {
@@ -445,30 +410,30 @@ ssl3_get_message(SSL *s, int st1, int stn, int mt, long max, int *ok)
 	if (SSL_is_dtls(s))
 		return (dtls1_get_message(s, st1, stn, mt, max, ok));
 
-	if (S3I(s)->tmp.reuse_message) {
-		S3I(s)->tmp.reuse_message = 0;
-		if ((mt >= 0) && (S3I(s)->tmp.message_type != mt)) {
+	if (S3I(s)->hs.tls12.reuse_message) {
+		S3I(s)->hs.tls12.reuse_message = 0;
+		if ((mt >= 0) && (S3I(s)->hs.tls12.message_type != mt)) {
 			al = SSL_AD_UNEXPECTED_MESSAGE;
 			SSLerror(s, SSL_R_UNEXPECTED_MESSAGE);
 			goto fatal_err;
 		}
 		*ok = 1;
-		s->internal->init_msg = s->internal->init_buf->data + 4;
-		s->internal->init_num = (int)S3I(s)->tmp.message_size;
+		s->internal->init_msg = s->internal->init_buf->data +
+		    SSL3_HM_HEADER_LENGTH;
+		s->internal->init_num = (int)S3I(s)->hs.tls12.message_size;
 		return s->internal->init_num;
 	}
 
 	p = (unsigned char *)s->internal->init_buf->data;
 
-	/* s->internal->init_num < 4 */
 	if (S3I(s)->hs.state == st1) {
 		int skip_message;
 
 		do {
-			while (s->internal->init_num < 4) {
-				i = s->method->internal->ssl_read_bytes(s,
+			while (s->internal->init_num < SSL3_HM_HEADER_LENGTH) {
+				i = s->method->ssl_read_bytes(s,
 				    SSL3_RT_HANDSHAKE, &p[s->internal->init_num],
-				    4 - s->internal->init_num, 0);
+				    SSL3_HM_HEADER_LENGTH - s->internal->init_num, 0);
 				if (i <= 0) {
 					s->internal->rwstate = SSL_READING;
 					*ok = 0;
@@ -489,51 +454,50 @@ ssl3_get_message(SSL *s, int st1, int stn, int mt, long max, int *ok)
 					s->internal->init_num = 0;
 					skip_message = 1;
 
-					if (s->internal->msg_callback)
-						s->internal->msg_callback(0, s->version,
-						    SSL3_RT_HANDSHAKE, p, 4, s,
-						    s->internal->msg_callback_arg);
+					ssl_msg_callback(s, 0,
+					    SSL3_RT_HANDSHAKE, p,
+					    SSL3_HM_HEADER_LENGTH);
 				}
 			}
 		} while (skip_message);
 
-		/* s->internal->init_num == 4 */
-
 		if ((mt >= 0) && (*p != mt)) {
 			al = SSL_AD_UNEXPECTED_MESSAGE;
 			SSLerror(s, SSL_R_UNEXPECTED_MESSAGE);
 			goto fatal_err;
 		}
 
-		CBS_init(&cbs, p, 4);
+		CBS_init(&cbs, p, SSL3_HM_HEADER_LENGTH);
 		if (!CBS_get_u8(&cbs, &u8) ||
 		    !CBS_get_u24(&cbs, &l)) {
 			SSLerror(s, ERR_R_BUF_LIB);
 			goto err;
 		}
-		S3I(s)->tmp.message_type = u8;
+		S3I(s)->hs.tls12.message_type = u8;
 
 		if (l > (unsigned long)max) {
 			al = SSL_AD_ILLEGAL_PARAMETER;
 			SSLerror(s, SSL_R_EXCESSIVE_MESSAGE_SIZE);
 			goto fatal_err;
 		}
-		if (l && !BUF_MEM_grow_clean(s->internal->init_buf, l + 4)) {
+		if (l && !BUF_MEM_grow_clean(s->internal->init_buf,
+		    l + SSL3_HM_HEADER_LENGTH)) {
 			SSLerror(s, ERR_R_BUF_LIB);
 			goto err;
 		}
-		S3I(s)->tmp.message_size = l;
+		S3I(s)->hs.tls12.message_size = l;
 		S3I(s)->hs.state = stn;
 
-		s->internal->init_msg = s->internal->init_buf->data + 4;
+		s->internal->init_msg = s->internal->init_buf->data +
+		    SSL3_HM_HEADER_LENGTH;
 		s->internal->init_num = 0;
 	}
 
 	/* next state (stn) */
 	p = s->internal->init_msg;
-	n = S3I(s)->tmp.message_size - s->internal->init_num;
+	n = S3I(s)->hs.tls12.message_size - s->internal->init_num;
 	while (n > 0) {
-		i = s->method->internal->ssl_read_bytes(s, SSL3_RT_HANDSHAKE,
+		i = s->method->ssl_read_bytes(s, SSL3_RT_HANDSHAKE,
 		    &p[s->internal->init_num], n, 0);
 		if (i <= 0) {
 			s->internal->rwstate = SSL_READING;
@@ -544,21 +508,14 @@ ssl3_get_message(SSL *s, int st1, int stn, int mt, long max, int *ok)
 		n -= i;
 	}
 
-	/* If receiving Finished, record MAC of prior handshake messages for
-	 * Finished verification. */
-	if (*s->internal->init_buf->data == SSL3_MT_FINISHED)
-		ssl3_take_mac(s);
-
 	/* Feed this message into MAC computation. */
 	if (s->internal->mac_packet) {
 		tls1_transcript_record(s, (unsigned char *)s->internal->init_buf->data,
-		    s->internal->init_num + 4);
+		    s->internal->init_num + SSL3_HM_HEADER_LENGTH);
 
-		if (s->internal->msg_callback)
-			s->internal->msg_callback(0, s->version,
-			    SSL3_RT_HANDSHAKE, s->internal->init_buf->data,
-			    (size_t)s->internal->init_num + 4, s,
-			    s->internal->msg_callback_arg);
+		ssl_msg_callback(s, 0, SSL3_RT_HANDSHAKE,
+		    s->internal->init_buf->data,
+		    (size_t)s->internal->init_num + SSL3_HM_HEADER_LENGTH);
 	}
 
 	*ok = 1;
@@ -566,7 +523,7 @@ ssl3_get_message(SSL *s, int st1, int stn, int mt, long max, int *ok)
 
  fatal_err:
 	ssl3_send_alert(s, SSL3_AL_FATAL, al);
-err:
+ err:
 	*ok = 0;
 	return (-1);
 }
@@ -594,7 +551,7 @@ ssl_cert_type(X509 *x, EVP_PKEY *pkey)
 		ret = SSL_PKEY_GOST01;
 	}
 
-err:
+ err:
 	if (!pkey)
 		EVP_PKEY_free(pk);
 	return (ret);
@@ -676,7 +633,7 @@ ssl3_setup_init_buffer(SSL *s)
 	s->internal->init_buf = buf;
 	return (1);
 
-err:
+ err:
 	BUF_MEM_free(buf);
 	return (0);
 }
@@ -716,7 +673,7 @@ ssl3_setup_read_buffer(SSL *s)
 	s->internal->packet = S3I(s)->rbuf.buf;
 	return 1;
 
-err:
+ err:
 	SSLerror(s, ERR_R_MALLOC_FAILURE);
 	return 0;
 }
@@ -749,7 +706,7 @@ ssl3_setup_write_buffer(SSL *s)
 
 	return 1;
 
-err:
+ err:
 	SSLerror(s, ERR_R_MALLOC_FAILURE);
 	return 0;
 }
